@@ -436,6 +436,8 @@ _restic_backup()
     local_nonpersistent_flags+=("--ignore-ctime")
     flags+=("--ignore-inode")
     local_nonpersistent_flags+=("--ignore-inode")
+    flags+=("--no-scan")
+    local_nonpersistent_flags+=("--no-scan")
     flags+=("--one-file-system")
     flags+=("-x")
     local_nonpersistent_flags+=("--one-file-system")
@@ -444,6 +446,10 @@ _restic_backup()
     two_word_flags+=("--parent")
     local_nonpersistent_flags+=("--parent")
     local_nonpersistent_flags+=("--parent=")
+    flags+=("--read-concurrency=")
+    two_word_flags+=("--read-concurrency")
+    local_nonpersistent_flags+=("--read-concurrency")
+    local_nonpersistent_flags+=("--read-concurrency=")
     flags+=("--stdin")
     local_nonpersistent_flags+=("--stdin")
     flags+=("--stdin-filename=")
@@ -1256,6 +1262,10 @@ _restic_generate()
     two_word_flags+=("--man")
     local_nonpersistent_flags+=("--man")
     local_nonpersistent_flags+=("--man=")
+    flags+=("--powershell-completion=")
+    two_word_flags+=("--powershell-completion")
+    local_nonpersistent_flags+=("--powershell-completion")
+    local_nonpersistent_flags+=("--powershell-completion=")
     flags+=("--zsh-completion=")
     two_word_flags+=("--zsh-completion")
     local_nonpersistent_flags+=("--zsh-completion")
@@ -2083,6 +2093,8 @@ _restic_restore()
     two_word_flags+=("--path")
     local_nonpersistent_flags+=("--path")
     local_nonpersistent_flags+=("--path=")
+    flags+=("--sparse")
+    local_nonpersistent_flags+=("--sparse")
     flags+=("--tag=")
     two_word_flags+=("--tag")
     local_nonpersistent_flags+=("--tag")
@@ -2139,6 +2151,106 @@ _restic_restore()
     noun_aliases=()
 }
 
+_restic_rewrite()
+{
+    last_command="restic_rewrite"
+
+    command_aliases=()
+
+    commands=()
+
+    flags=()
+    two_word_flags=()
+    local_nonpersistent_flags=()
+    flags_with_completion=()
+    flags_completion=()
+
+    flags+=("--dry-run")
+    flags+=("-n")
+    local_nonpersistent_flags+=("--dry-run")
+    local_nonpersistent_flags+=("-n")
+    flags+=("--exclude=")
+    two_word_flags+=("--exclude")
+    two_word_flags+=("-e")
+    local_nonpersistent_flags+=("--exclude")
+    local_nonpersistent_flags+=("--exclude=")
+    local_nonpersistent_flags+=("-e")
+    flags+=("--exclude-file=")
+    two_word_flags+=("--exclude-file")
+    local_nonpersistent_flags+=("--exclude-file")
+    local_nonpersistent_flags+=("--exclude-file=")
+    flags+=("--forget")
+    local_nonpersistent_flags+=("--forget")
+    flags+=("--help")
+    flags+=("-h")
+    local_nonpersistent_flags+=("--help")
+    local_nonpersistent_flags+=("-h")
+    flags+=("--host=")
+    two_word_flags+=("--host")
+    two_word_flags+=("-H")
+    local_nonpersistent_flags+=("--host")
+    local_nonpersistent_flags+=("--host=")
+    local_nonpersistent_flags+=("-H")
+    flags+=("--iexclude=")
+    two_word_flags+=("--iexclude")
+    local_nonpersistent_flags+=("--iexclude")
+    local_nonpersistent_flags+=("--iexclude=")
+    flags+=("--iexclude-file=")
+    two_word_flags+=("--iexclude-file")
+    local_nonpersistent_flags+=("--iexclude-file")
+    local_nonpersistent_flags+=("--iexclude-file=")
+    flags+=("--path=")
+    two_word_flags+=("--path")
+    local_nonpersistent_flags+=("--path")
+    local_nonpersistent_flags+=("--path=")
+    flags+=("--tag=")
+    two_word_flags+=("--tag")
+    local_nonpersistent_flags+=("--tag")
+    local_nonpersistent_flags+=("--tag=")
+    flags+=("--cacert=")
+    two_word_flags+=("--cacert")
+    flags+=("--cache-dir=")
+    two_word_flags+=("--cache-dir")
+    flags+=("--cleanup-cache")
+    flags+=("--compression=")
+    two_word_flags+=("--compression")
+    flags+=("--insecure-tls")
+    flags+=("--json")
+    flags+=("--key-hint=")
+    two_word_flags+=("--key-hint")
+    flags+=("--limit-download=")
+    two_word_flags+=("--limit-download")
+    flags+=("--limit-upload=")
+    two_word_flags+=("--limit-upload")
+    flags+=("--no-cache")
+    flags+=("--no-lock")
+    flags+=("--option=")
+    two_word_flags+=("--option")
+    two_word_flags+=("-o")
+    flags+=("--pack-size=")
+    two_word_flags+=("--pack-size")
+    flags+=("--password-command=")
+    two_word_flags+=("--password-command")
+    flags+=("--password-file=")
+    two_word_flags+=("--password-file")
+    two_word_flags+=("-p")
+    flags+=("--quiet")
+    flags+=("-q")
+    flags+=("--repo=")
+    two_word_flags+=("--repo")
+    two_word_flags+=("-r")
+    flags+=("--repository-file=")
+    two_word_flags+=("--repository-file")
+    flags+=("--tls-client-cert=")
+    two_word_flags+=("--tls-client-cert")
+    flags+=("--verbose")
+    flags+=("-v")
+
+    must_have_one_flag=()
+    must_have_one_noun=()
+    noun_aliases=()
+}
+
 _restic_self-update()
 {
     last_command="restic_self-update"
@@ -2617,6 +2729,7 @@ _restic_root_command()
     commands+=("rebuild-index")
     commands+=("recover")
     commands+=("restore")
+    commands+=("rewrite")
     commands+=("self-update")
     commands+=("snapshots")
     commands+=("stats")
