@@ -26,12 +26,12 @@ func newTestSuite() *test.Suite {
 		Create: func(cfg interface{}) (restic.Backend, error) {
 			c := cfg.(*memConfig)
 			if c.be != nil {
-				ok, err := c.be.Test(context.TODO(), restic.Handle{Type: restic.ConfigFile})
-				if err != nil {
+				_, err := c.be.Stat(context.TODO(), restic.Handle{Type: restic.ConfigFile})
+				if err != nil && !c.be.IsNotExist(err) {
 					return nil, err
 				}
 
-				if ok {
+				if err == nil {
 					return nil, errors.New("config already exists")
 				}
 			}
