@@ -6,7 +6,7 @@ import (
 	"os"
 	"path"
 
-	"github.com/restic/restic/internal/backend"
+	"github.com/restic/restic/internal/backend/layout"
 	"github.com/restic/restic/internal/backend/s3"
 	"github.com/restic/restic/internal/cache"
 	"github.com/restic/restic/internal/debug"
@@ -38,19 +38,19 @@ func toS3Backend(repo restic.Repository) *s3.Backend {
 }
 
 // Check tests whether the migration can be applied.
-func (m *S3Layout) Check(ctx context.Context, repo restic.Repository) (bool, error) {
+func (m *S3Layout) Check(ctx context.Context, repo restic.Repository) (bool, string, error) {
 	be := toS3Backend(repo)
 	if be == nil {
 		debug.Log("backend is not s3")
-		return false, nil
+		return false, "backend is not s3", nil
 	}
 
 	if be.Layout.Name() != "s3legacy" {
 		debug.Log("layout is not s3legacy")
-		return false, nil
+		return false, "not using the legacy s3 layout", nil
 	}
 
-	return true, nil
+	return true, "", nil
 }
 
 func (m *S3Layout) RepoCheck() bool {
@@ -74,7 +74,7 @@ func retry(max int, fail func(err error), f func() error) error {
 // maxErrors for retrying renames on s3.
 const maxErrors = 20
 
-func (m *S3Layout) moveFiles(ctx context.Context, be *s3.Backend, l backend.Layout, t restic.FileType) error {
+func (m *S3Layout) moveFiles(ctx context.Context, be *s3.Backend, l layout.Layout, t restic.FileType) error {
 	printErr := func(err error) {
 		fmt.Fprintf(os.Stderr, "renaming file returned error: %v\n", err)
 	}
@@ -97,12 +97,12 @@ func (m *S3Layout) Apply(ctx context.Context, repo restic.Repository) error {
 		return errors.New("backend is not s3")
 	}
 
-	oldLayout := &backend.S3LegacyLayout{
+	oldLayout := &layout.S3LegacyLayout{
 		Path: be.Path(),
 		Join: path.Join,
 	}
 
-	newLayout := &backend.DefaultLayout{
+	newLayout := &layout.DefaultLayout{
 		Path: be.Path(),
 		Join: path.Join,
 	}
